PROGRAM Underground;

#include "str.h"

CONST probname = 'NZ93_';
      prob_id = 'B';
      file_id = 'PROBLEM';
VAR   debugcounter: integer;

CONST MaxStation = 675;  {26*26 - 1}

TYPE  Station_index = -1..MaxStation;
      Junction_index = 0..100; {probably enough}
      RouteChar = 'A'..'Z';
      StationChar = 'a'..'z';
      Tstatus = (unseen, fringe, seen);

VAR   Network: array[Station_index] of record
                 prev,next,father: Station_index;
                 status: Tstatus;
                 J_no: integer {Junction_index} end;
      ThisRoute: RouteChar;
      GoodRoutes, GoodStations: set of char;
      Used: array[RouteChar] of Boolean;
      junctions: array[Junction_index, 1..26] of Station_index;
      Jnum: Junction_index; i: integer;
      begStation, endStation: Station_index;

PROCEDURE ReadRoutes;            forward;
FUNCTION  ReadData: Boolean;     forward;
PROCEDURE SolveProblem;          forward;

PROCEDURE Error(s:string);
begin
  writeln; write(s, '... ' ); readln; HALT;
end;

PROCEDURE MainProg;
begin  {Main}

GoodRoutes := ['A'..'Z']; GoodStations := ['a'..'z'];
for ThisRoute := 'A' to 'Z' do Used[ThisRoute] := false;
for Jnum := 0 to 100 do for i := 1 to 26 do junctions[Jnum, i] := -1;
Jnum := 0;

ReadRoutes;
while ReadData do
  SolveProblem;

end; {of main procedure}

PROCEDURE NumToChars(Num: Station_index; var ch1, ch2: char);
begin
ch1 := chr((Num div 26) + ord('A'));
ch2 := chr((Num mod 26) + ord('a'));
end;

FUNCTION CharsToNum(ch1, ch2: char): Station_index;
begin CharsToNum := 26*(ord(ch1) - ord('A')) + (ord(ch2) - ord('a')) end;

PROCEDURE PrintRoutes;
VAR ThisRoute, ThisStation: char;
    Route_id: integer;
    i,j: integer;
    ch1, ch2: char;

begin
For ThisRoute:= 'A' to 'Z' do
  if used[ThisRoute] then begin
    writeln('Route: ', ThisRoute);
    Route_id := 26 * (ord(ThisRoute) - ord('A'));
    for ThisStation := 'a' to 'z' do
      with Network[Route_id + ord(ThisStation) - ord('a')] do
        if (next <> -1) or (prev <> -1)
          then begin
            NumToChars(prev, ch1, ch2);
            write(ch1, ' <- ', ThisStation, ' -> ', ch2);
            if J_no <> maxint
              then writeln(J_no)
              else writeln;
            end;
    readln end;

for i := 1 to 100 do
  if junctions[i, 1] <> -1
    then begin
      write(' Junction ', i:2, ': '); j := 1;
      repeat
        NumToChars(junctions[i,j], ch1, ch2);
        write(ch1, ch2, ',');
        inc(j) until junctions[i,j] = -1;
      writeln end;
readln;
end;  {Print Routes}

PROCEDURE CheckRoute(RouteName: char);
begin
if not(RouteName in GoodRoutes)
  then Error(RouteName + ' is not a good route designator');
end;

PROCEDURE ReadRoutes;
var   i, NumRoutes, Route_id: integer;
      RouteName, ch: char;
      Last_Station, This_Station: Station_index;

PROCEDURE AddStation;
begin
This_Station := Route_id*26 + (ord(ch) - ord('a'));
Network[Last_Station].next := This_Station;
Network[This_Station].prev := Last_Station;
Last_Station := This_Station;
end;  {New Station}

PROCEDURE AddJunction(s1,s2: Station_index);
begin
inc(Jnum); junctions[Jnum,1] := s1; junctions[Jnum,2] := s2;
Network[s1].J_no := Jnum; Network[s2].J_no := Jnum;
end;  {Add Junction}

PROCEDURE CombineJunctions(j1,j2: junction_index);
var i,j: integer;

begin
i := 1; while junctions[j1,i] <> -1 do inc(i);
j := 1;
repeat
  Network[junctions[j2,j]].J_no := j1;
  junctions[j1,i] := junctions[j2, j];
  inc(i); inc(j) until junctions[j2,j] = -1;
junctions[j2,1] := -1;
end;

PROCEDURE DoJunction;
var JunctionRoute, JunctionStat: char;
    Junction_id: Station_index;
    j1, j2: integer;
    ch1, ch2: char;

begin
read(JunctionRoute, JunctionStat);
CheckRoute(JunctionRoute);
if not (JunctionStat in GoodStations)
  then Error(JunctionStat + ' is not a good station in Junction');
Junction_id := 26 * (ord(JunctionRoute) - ord('A'))
                  + (ord(JunctionStat) - ord('a'));

j1 := Network[This_Station].J_no; j2 := Network[Junction_id].J_no;
if (j1 = maxint) AND (j2 = maxint)
  then AddJunction(This_Station, Junction_id) else
if (j1 = maxint)
  then Network[This_station].J_no := j2 else
if (j2 = maxint)
  then Network[Junction_id].J_no := j1 else
if (j1 = j2)
  then {Do nothing} else
if (j1 < j2)
  then CombineJunctions(j1, j2)
  else CombineJunctions(j2, j1);
end;  {Dealing with alias}

begin  {Read Routes}
for This_Station := 0 to MaxStation do
  with Network[This_Station] do begin
    prev := -1; next := -1; J_no := maxint end;
readln(NumRoutes);
for i := 1 to NumRoutes do begin
  read(RouteName, ch);
  CheckRoute(RouteName);
  if Used[RouteName]
    then Error('Duplicate Route ' + RouteName);
  if ch <> ':'
    then Error(ch + ' != :');
  Used[RouteName] := true; Route_id := ord(RouteName) - ord('A');
  Last_Station := -1;
  while not eoln do begin
    read(ch);
    if ch in GoodStations
      then AddStation else
    if ch = '='
      then DoJunction
      else Error(ch + ' is not a good station designator');
    end;  {while not eoln}
  readln end;  {for i := 1 to Num Routes}
end;  {Read Routes}

function ReadData: Boolean;
var line: string;

begin
{PrintRoutes;}
readln(line);
if line = '#'
  then ReadData := false
  else begin
    ReadData := true;
    endStation := CharsToNum(line[1], line[2]);
    begStation := CharsToNum(line[3], line[4]);
    end;
end;

PROCEDURE SolveProblem;
TYPE ptr = ^entry;
     entry = record
               me: Station_Index;
               cost: integer;
               next: ptr end;

VAR  ch1, ch2, Route: char;
     Q_head, Q_tail: ptr;
     heap: ptr;
     This_Station: Station_index;
     cost: integer;
     j1, j2: integer;

PROCEDURE Push(This_Station, Last_Station: Station_index;
               This_cost: integer);
LABEL 999;

VAR p, q: ptr;

begin
if (This_Station = -1) OR (Network[This_Station].status <> unseen) then goto 999;

with Network[This_Station] do begin
  father := Last_Station; status := fringe
{**;writeln('this_station, last_station, cost', This_station:6, Last_Station:6, This_cost:6); readln;{**}
   end;
new(p);
with p^ do begin
  me := This_Station; cost := This_cost end;

if Q_head = nil
  then begin
    new(Q_tail);
    with Q_tail^ do begin me := -1; cost := maxint end;
    Q_head := p; Q_head^.next := Q_tail end
  else begin
    if This_cost < Q_Head^.cost
      then begin p^.next := Q_head; Q_head := p end
      else begin
        q := Q_head; while This_cost > q^.next^.cost do q := q^.next;
        p^.next := q^.next; q^.next := p end;
        end;
999: end;  {Push}

PROCEDURE Pop(var This_Station: Station_index; var This_cost: integer);
begin
with Q_head^ do begin
  This_Station := me; This_cost := cost end;
Q_head := Q_head^.next;
end;

begin  {Solve Problem}
(*Mark(heap);*)
for This_Station := 0 to 675 do
  with Network[This_Station] do begin status := unseen; father := -1 end;
Q_head := nil; Q_tail := nil;
Push(begStation, -1, 0); This_Station := begStation;
while Q_head^.me <> endStation do begin
  pop(This_Station, cost);
  with Network[This_Station] do begin
    status := seen;
    Push(prev, This_Station, cost+1); Push(next, This_Station, cost+1);
    j1 := J_no; j2 := 1;
    if j1 <> maxint
      then while junctions[j1, j2] <> -1 do begin
        if Network[junctions[j1, j2]].status = unseen
          then Push(junctions[j1, j2], This_Station, cost+3); inc(j2) end;
    end;
  end;

write(Q_head^.cost:3, ': ');
NumToChars(endStation, Route, ch2); write(Route, ch2);
repeat
endStation := Network[endStation].father;
NumToChars(endStation, ch1, ch2);
if ch1 = Route
  then write(ch2)
  else begin Route := ch1;write('=', Route, ch2) end;
until endStation = begStation;
writeln;
(*Release(heap);*)
end;  {Solve Problem}

begin mainprog end.